module AudioFeedManager
  describe ArgumentsParser do
    let(:parser) { isolate(ArgumentsParser) }

    context "with a simple one-argument specification" do
      let(:specification) {
        ArgumentsSpecification.specify do |s|
          s.argument :project_name, required: true, description: "Name of the directory to create the project in"
        end
      }

      it "parses a single-argument list into arguments" do
        arguments = parser.parse(specification, ["foo-bar"])

        expect(arguments.to_hash).to eq({project_name: "foo-bar"})
      end

      it "fails when there are no arguments" do
        expect {
          parser.parse(specification, [])
        }.to raise_error(InvalidArguments)
      end

      it "fails when there are two or more arguments" do
        expect {
          parser.parse(specification, ["foo", "bar"])
        }.to raise_error(InvalidArguments)
      end
    end

    context "with varargs" do
      let(:specification) {
        ArgumentsSpecification.specify do |s|
          s.varargs :files, minimum: 1, description: "Files to add."
        end
      }

      it "parses a single-argument list into arguments" do
        arguments = parser.parse(specification, ["foo-bar"])

        expect(arguments.to_hash).to eq({files: ["foo-bar"]})
      end

      it "parses a multi-argument list into arguments" do
        arguments = parser.parse(specification, ["foo", "bar", "baz"])

        expect(arguments.to_hash).to eq({files: ["foo", "bar", "baz"]})
      end

      it "fails when there are no arguments" do
        expect {
          parser.parse(specification, [])
        }.to raise_error(InvalidArguments)
      end
    end

    context "with optional arguments" do
      let(:specification) {
        ArgumentsSpecification.specify do |s|
          s.argument :author_name, required: false, default: "Anonymous", description: "Author name"
          s.argument :project_name, required: true, description: "Name of the directory to create the project in"
        end
      }

      it "parses a single-argument list into required arguments" do
        arguments = parser.parse(specification, ["foo-bar"])

        expect(arguments.to_hash[:project_name]).to eq("foo-bar")
      end

      it "parses a two-argument list into required and optional arguments" do
        arguments = parser.parse(specification, ["foo", "bar"])

        expect(arguments.to_hash[:project_name]).to eq("foo")
        expect(arguments.to_hash[:author_name]).to eq("bar")
      end

      it "raises an error when given more arguments than supported" do
        expect {
          parser.parse(specification, ["foo", "bar", "baz"])
        }.to raise_error(InvalidArguments)
      end
    end
  end
end
